@@ -1,6 +1,6 @@
 ;;; browse-url.el --- pass a URL to a WWW browser
 
-;; Copyright (C) 1995, 96, 97, 98, 99, 2000, 2001, 2002
+;; Copyright (C) 1995, 96, 97, 98, 99, 2000, 2001, 2002, 2003
 ;;   Free Software Foundation, Inc.
 
 ;; Author: Denis Howe <dbh@doc.ic.ac.uk>
@@ -25,6 +25,8 @@
 ;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 ;; Boston, MA 02111-1307, USA.
 
+;;; Synched up with:  Version 1.32 in CVS
+
 ;;; Commentary:
 
 ;; This package provides functions which read a URL (Uniform Resource
@@ -39,6 +41,7 @@
 ;; Function                           Browser     Earliest version
 ;; browse-url-mozilla                 Mozilla     Don't know
 ;; browse-url-galeon                  Galeon      Don't know
+;; browse-url-epiphany                Epiphany    Don't know
 ;; browse-url-netscape                Netscape    1.1b1
 ;; browse-url-mosaic                  XMosaic/mMosaic <= 2.4
 ;; browse-url-cci                     XMosaic     2.5
@@ -52,6 +55,7 @@
 ;; browse-url-mmm                     MMM         ?
 ;; browse-url-generic                 arbitrary
 ;; browse-url-default-windows-browser MS-Windows browser
+;; browse-url-default-macosx-browser  Mac OS X browser
 ;; browse-url-gnome-moz               GNOME interface to Mozilla
 ;; browse-url-kde                     KDE konqueror (kfm)
 ;; browse-url-opera                   Opera
@@ -107,8 +111,8 @@
 ;; <URL:http://www.unlv.edu/chimera/>, Arena
 ;; <URL:ftp://ftp.yggdrasil.com/pub/dist/web/arena> and Amaya
 ;; <URL:ftp://ftp.w3.org/pub/amaya>.  mMosaic
-;; <URL:ftp://sig.enst.fr/pub/multicast/mMosaic/>,
-;; <URL:http://sig.enst.fr/~dauphin/mMosaic/> (with development
+;; <URL:ftp://ftp.enst.fr/pub/mbone/mMosaic/>,
+;; <URL:http://www.enst.fr/~dauphin/mMosaic/> (with development
 ;; support for Java applets and multicast) can be used like Mosaic by
 ;; setting `browse-url-mosaic-program' appropriately.
 
@@ -222,329 +226,6 @@
 ;;      (setq browse-url-browser-function '(("^mailto:" . browse-url-mail)
 ;;      				    ("." . browse-url-netscape)))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; Change Log:
-;;
-;; 0.00 03 Apr 1995 Denis Howe <dbh@doc.ic.ac.uk>
-;;	Created.
-;; 0.01 04 Apr 1995
-;;	All names start with "browse-url-".  Added provide.
-;; 0.02 05 Apr 1995
-;;	Save file at start of browse-url-of-file.
-;;	Use start-process instead of start-process-shell-command.
-;; 0.03 06 Apr 1995
-;;	Add browse-url-netscape-reload, browse-url-netscape-send.
-;;	browse-url-of-file save file option.
-;; 0.04 08 Apr 1995
-;;	b-u-file-url separate function.  Change b-u-filename-alist
-;;	default.
-;; 0.05 09 Apr 1995
-;;	Added b-u-of-file-hook.
-;; 0.06 11 Apr 1995
-;;	Improved .emacs suggestions and documentation.
-;; 0.07 13 Apr 1995
-;;	Added browse-url-interactive-arg optional prompt.
-;; 0.08 18 Apr 1995
-;;	Exclude final "." from browse-url-regexp.
-;; 0.09 21 Apr 1995
-;;	Added mouse-set-point to browse-url-interactive-arg.
-;; 0.10 24 Apr 1995
-;;	Added Mosaic signal sending variations.
-;;	Thanks Brian K Servis <servis@ecn.purdue.edu>.
-;;	Don't use xprop for Netscape.
-;; 0.11 25 Apr 1995
-;;	Fix reading of ~/.mosaicpid.  Thanks Dag.H.Wanvik@kvatro.no.
-;; 0.12 27 Apr 1995
-;;	Interactive prefix arg => URL *after* point.
-;;	Thanks Michelangelo Grigni <mic@cs.ucsd.edu>.
-;;	Added IXI Mosaic support.
-;;	Thanks David Karr <dkarr@nmo.gtegsc.com>.
-;; 0.13 28 Apr 1995
-;;	Exclude final [,;] from browse-url-regexp.
-;; 0.14 02 May 1995
-;;	Provide browser argument variables.
-;; 0.15 07 May 1995
-;;	More Netscape options.  Thanks Peter Arius
-;;	<arius@immd2.informatik.uni-erlangen.de>.
-;; 0.16 17 May 1995
-;;	Added browse-url-at-mouse.
-;;	Thanks Wayne Mesard <wmesard@sgi.com>
-;; 0.17 27 Jun 1995
-;;	Renamed browse-url-at-point to browse-url-url-at-point.
-;;	Added browse-url-at-point.
-;;	Thanks Jonathan Cano <cano@patch.tandem.com>.
-;; 0.18 16 Aug 1995
-;;	Fixed call to browse-url-url-at-point in browse-url-at-point.
-;;	Thanks Eric Ding <ericding@San-Jose.ate.slb.com>.
-;; 0.19 24 Aug 1995
-;;	Improved documentation.
-;;	Thanks Kevin Rodgers <kevin.rodgers@ihs.com>.
-;; 0.20 31 Aug 1995
-;;	browse-url-of-buffer to handle file-less buffers.
-;;	browse-url-of-dired-file browses current file in dired.
-;;	Thanks Kevin Rodgers <kevin.rodgers@ihs.com>.
-;; 0.21 09 Sep 1995
-;;	XMosaic CCI functions.
-;;	Thanks Marc Furrer <Marc.Furrer@di.epfl.ch>.
-;; 0.22 13 Sep 1995
-;;	Fixed new-window documentation and added to browse-url-cci.
-;;	Thanks Dilip Sequeira <djs@dcs.ed.ac.uk>.
-;; 0.23 10 Nov 1995
-;;	Added b-u-lynx.  Thanks Steven L. Baur <steve@xemacs.org>.
-;; 0.24 22 Nov 1995
-;;	Renamed b-u-netscape command to b-u-netscape-send.
-;;	Added b-u-netscape-command variable.
-;; 0.25 03 Dec 1995
-;;	Added event-buffer and event-point for XEmacs compatibility.
-;;	Thanks Eric Engstrom <engstrom@src.honeywell.com>
-;; 0.26 13 Jan 1996
-;;	Changed b-u-lynx to b-u-lynx-xterm, added b-u-lynx-emacs to
-;;	run Lynx in an Emacs buffer under terminal-emulator.
-;;	Thanks Jari Aalto <jaalto@tre.tele.nokia.fi>
-;; 0.27 27 Feb 1996
-;;	Changed event-buffer and event-point from macros to functions.
-;;	Other fixes for byte-compilation.
-;; 0.28 07 Mar 1996
-;;	browse-url-lynx-emacs uses term.el instead of terminal.el.
-;; 0.29 13 Mar 1996
-;;	Added browse-url-CCI-host.  Thanks Greg Marr <gregm@WPI.EDU>.
-;; 0.30 23 Mar 1996
-;;	Contact/start Netscape in the background.
-;;	Thanks Per Abrahamsen <abraham@dina.kvl.dk>
-;; 0.31 28 Apr 1996
-;;	Added browse-url command.
-;;	Added new-window logic to b-u-interactive-arg.
-;;	b-u-file-url checks for EFS path.
-;; 0.32 02 May 1996
-;;	Improved b-u-url-at-point matching to supply missing "http://".
-;; 0.33 01 Jun 1996
-;;	Jari Aalto <jaalto@tre.tele.nokia.fi> browse-url-lynx-emacs
-;;	fix.  Thanks Jari.
-;; 0.34 05 Jun 1996
-;;	b-u-file-url checks for EFS after alist.  Thanks
-;;	Jens-U H Petersen <petersen@kurims.kyoto-u.ac.jp>
-;; 0.35 11 Jun 1996
-;;	Grail support.  Thanks Barry A. Warsaw
-;;	<bwarsaw@anthem.cnri.reston.va.us>.
-;; 0.36 12 Jun 1996
-;;	Fixed browse-url-looking-at (I hope).
-;; 0.37 15 Jun 1996
-;;	b-u-file-url URL-encodes special chars.
-;;	Thanks Martin Schwenke <Martin.Schwenke@cs.anu.edu.au>.
-;; 0.38 17 Jun 1996
-;;	b-u-file-url encodes fewer chars.  Multi-display support for
-;;	Netscape.  Thanks Richard Mlynarik <mly@adoc.xerox.com>
-
-;; 1996-10-10 Denis Howe <dbh@doc.ic.ac.uk>
-;;      * browse-url.el (browse-url): New function.
-;;      (browse-url-CCI-host): New variable.
-;;      (browse-url-at-mouse): Added event-buffer and event-point
-;;      functions for XEmacs compatibility.
-;;      (browse-url-file-url): Check for EFS after alist, URL-encode
-;;      special chars.
-;;      (browse-url-grail): New function.
-;;      (browse-url-interactive-arg): Add new-window logic.
-;;      (browse-url-looking-at): Fixed.
-;;      (browse-url-lynx-xterm): New function.
-;;      (browse-url-lynx-emacs): Use term.el instead of terminal.el.
-;;      (browse-url-netscape): Contact/start Netscape in the
-;;      background.  Multi-display support.  Renamed
-;;      browse-url-netscape-send.  URL-encode comma.
-;;      (browse-url-netscape-command): New variable.
-;;      (browse-url-netscape-startup-arguments): New variable.
-;;      (browse-url-url-at-point): Improved matching to supply missing
-;;      "http://".
-;;      Other fixes for byte-compilation.
-;;1996-10-10  Richard Stallman  <rms@ethanol.gnu.ai.mit.edu>
-;;      * browse-url.el (browse-url-at-point): Add autoload cookie.
-;;      (browse-url-at-mouse, browse-url-of-buffer, browse-url-of-file)
-;;      (browse-url-of-dired-file): Likewise.
-;;1996-12-11  Richard Stallman  <rms@psilocin.gnu.ai.mit.edu>
-;;      * browse-url.el (browse-url-url-at-point): Use thing-at-point.
-;;      (browse-url-looking-at): Function deleted.
-;;1997-04-14  Karl Heuer  <kwzh@gnu.ai.mit.edu>
-;;      * browse-url.el (browse-url-browser-function): Fix syntax error.
-;;1997-05-14  Richard Stallman  <rms@psilocin.gnu.ai.mit.edu>
-;;      * browse-url.el (browse-url-event-buffer):
-;;      Function renamed from event-buffer.  Callers changed.
-;;      (browse-url-event-point): Likewise.
-;;      * browse-url.el: Don't include term.el.
-;;1997-05-15  Richard Stallman  <rms@psilocin.gnu.ai.mit.edu>
-;;      * browse-url.el (browse-url): Make `applicatations' the parent.
-;;1997-05-27  Richard Stallman  <rms@psilocin.gnu.ai.mit.edu>
-;;      * browse-url.el (browse-url-netscape, browse-url-netscape-sentinel)
-;;      (browse-url-netscape-send): Use browse-url-netscape-program,
-;;      renamed from browse-url-netscape-command.
-;;1997-06-08  Dave Love  <d.love@dl.ac.uk>
-;;      * browse-url.el: Update keywords to show up in finder.
-;;      (browse-url-gnudoit-args, browse-url-generic-program)
-;;      (browse-url-gnudoit-program, browse-url-generic-args): New variables.
-;;      (browse-url-w3-gnudoit): New procedure.
-;;      (browse-url-mmm): New location of `remote' file for MMM 0.4.
-;;      (browse-url-generic): New procedure.
-
-;;      (browse-url-netscape): Test for w32.
-
-;;      (browse-url-url-at-point): Assume mailto: if URL contains @.
-;;      Don't use thingatpt; find the URL here to do it correctly.
-
-;;      (browse-url-at-point, browse-url-of-file, browse-url-at-mouse):
-;;      Call browse-url.
-;;      (browse-url): Check for list browse-url-browser-function.
-;;      (browse-url-choose-browser): New procedure.
-;;      (browse-url-browser-function): Allow list value.
-
-;;      (browse-url-process-environment): Call browse-url-emacs-display.
-;;      (browse-url-emacs-display): New procedure.
-;;      (browse-url-netscape-display): New variable.
-
-;;      (browse-url-of-region): New procedure.
-;;      (browse-url-of-buffer): Check for narrowed buffer.
-
-;;      (browse-url-url-at-point): Rewrite to not use cl.el delete-if.
-;;      Fix multi-line URL matching.
-
-;;      (browse-url-markedup-regexp): New variable.
-;;      (browse-url-xterm-program): New variable.
-;;      (browse-url-xterm-args): New variable.
-;;      (browse-url-lynx-xterm): Use the above two vars.
-;;      (browse-url-url-at-point): Use buffer-substring-no-properties.
-;;      (browse-url-grail): Add missing optional arg.
-;;      (browse-url-mmm): New procedure.
-
-;;      (browse-url-netscape-startup-arguments): New variable.
-
-;;1997-07-04  Dave Love  <d.love@dl.ac.uk>
-;;      * browse-url.el: Require thingatpt when compiling.
-;;      (browse-url-url-at-point): Use `thing-at-point' (with URL code
-;;      moved from here).
-;;      (browse-url-looking-at): Moved to thingatpt.el, renamed and changed.
-;;1997-08-20  Dave Love  <d.love@dl.ac.uk>
-;;      * browse-url.el: Minor doc fixes.
-;;      (browse-url-temp-dir): New variable.
-;;      (browse-url-of-buffer): Use browse-url-temp-dir.  Don't construct
-;;      a temporary file name which includes an arbitrary buffer name to
-;;      avoid losing on non-unixy systems.
-;;1997-08-27  Dave Love  <d.love@dl.ac.uk>
-;;      * browse-url.el (browse-url-mail): Use compose-mail[-other-window],
-;;      not always `mail'.
-;;1997-08-27  Dave Love  <d.love@dl.ac.uk>
-;;      * browse-url.el: Change the custom group to `hypermedia'.
-;;      (browse-url-netscape-version): New variable.
-;;      (browse-url-netscape-reload): Use it to account for reported
-;;      backwards incompatibility.
-;;1997-09-02  Geoff Voelker  <voelker@cs.washington.edu>
-;;      * browse-url.el (browse-url-netscape): Check for w32 window-system
-;;      symbol.
-;;1997-09-08  Andreas Schwab  <schwab@issan.informatik.uni-dortmund.de>
-;;      * browse-url.el (browse-url group): Remove autoload cookie.
-;;1997-10-01  Andreas Schwab  <schwab@issan.informatik.uni-dortmund.de>
-;;      * browse-url.el (browse-url-netscape-display): Likewise.
-;;      (browse-url-generic-program): Likewise.
-;;1997-10-23  Dave Love  <d.love@dl.ac.uk>
-;;      * browse-url.el: Doc fixes.
-;;      (browse-url): Apply, don't just call, browse-url-choose-browser,
-;;      so args works properly.
-;;      (browse-url-maybe-new-window): New function.
-;;      (browse-url-netscape, browse-url-mosaic, browse-url-cci,
-;;      browse-url-w3, browse-url-lynx-emacs, browse-url-mail): Use it.
-;;      (browse-url-lynx-emacs): Don't call term-term-name.
-;;      (browse-url-lynx-input-field, browse-url-lynx-input-delay,
-;;      browse-url-lynx-input-attempts): New variables.
-;;      (browse-url-lynx-emacs): Use any existing Lynx buffer; take care
-;;      to move off input fields.  (After Vladimir Alexiev
-;;      <vladimir@cs.ualberta.ca>.)
-;;      (browse-url-mosaic-program): New variable.
-;;      (browse-url-mosaic): Use browse-url-mosaic-program and
-;;      browse-url-new-window-p.
-;;      (browse-url-at-mouse): Use browse-url-new-window-p.
-;;1997-11-21  Dave Love  <d.love@dl.ac.uk>
-;;      * browse-url.el: Doc fixes and extra customization.
-;;      (browse-url): Quote browse-url-choose-browser in application.
-;;1997-12-12  Dave Love  <d.love@dl.ac.uk>
-;;      * browse-url.el (browse-url-lynx-emacs): Add sentinel to kill the
-;;      buffer when lynx exits.  Doc fix.
-;;      (browse-url-browser-function): Better customization.
-;;      (browse-url-filename-alist, browse-url-netscape-display,
-;;      browse-url-filename-alist, browse-url-generic-program): Likewise.
-;;      (browse-url-new-window-p, browse-url-w3, browse-url-mail): Doc fix.
-;;      (browse-url-save-file): Customize.
-;;1997-12-22  Dave Love  <d.love@dl.ac.uk>
-;;      * browse-url.el (browse-url-path-regexp, browse-url-short-regexp)
-;;      (browse-url-regexp): Variables deleted (moved to thingatpt.el).
-;;      (browse-url-filename-alist): Default now has leading ange-ftp-type
-;;      patterns so that the "^/+" pattern doesn't mung such names in
-;;      advance.
-;;      (browse-url-file-url): Don't do explicit ange-ftp path munging here.
-;;      (browse-url-netscape-program): Doc addition.
-;;1998-03-06  Dave Love  <d.love@dl.ac.uk>
-;;      * browse-url.el Various doc fixes, mainly to remove innappropriate
-;;      leading "*"s.
-;;      (browse-url-new-window-p, browse-url-netscape-display)
-;;      (browse-url-save-file, browse-url-generic-program):
-;;      Add autoload cookies.
-;;      (browse-url-mosaic-program, browse-url-lynx-input-field)
-;;      (browse-url-lynx-input-attempts, browse-url-lynx-input-delay):
-;;      Add :version.
-;;      (browse-url-of-file-hook): Add :options.
-;;      (browse-url-lynx-emacs-args): New option.
-;;      (browse-url-lynx-emacs): Use it.
-;;      (browse-url-lynx-xterm): Use backquote.
-;;      (browse-url-emacs-display): Use Emacs version, not XEmacs.
-;;      ((require 'term)): When compiling.
-;;1998-05-15  Geoff Voelker  <voelker@cs.washington.edu>
-;;      * browse-url.el (browse-url-temp-dir): Use system-tmp-directory.
-;;1998-05-16  Richard Stallman  <rms@psilocin.ai.mit.edu>
-;;      * browse-url.el (browse-url-temp-dir): Use temporary-file-directory.
-;;1998-06-10  Dave Love  <fx@gnu.org>
-;;      * browse-url.el: Don't require dired when compiling.
-;;      (browse-url-lynx-input-delay, browse-url-lynx-input-attempts):
-;;      Remove customization.
-;;      (browse-url-grail): Respect new-window arg (Barry Warsaw).
-;;1998-06-19  Dave Love  <fx@gnu.org>
-;;      * browse-url.el (browse-url-netscape): Encode `)' too.
-;;1998-11-12  Sam Steingold  <sds@goems.com>
-;;      * browse-url.el (browse-url): Handle the case when
-;;      `browse-url-browser-function' is a lambda list.
-;;1998-12-09  Dave Love  <fx@gnu.org>
-;;      * browse-url.el (browse-url-mail): Call compose-mail-other-window
-;;      with correct number of args.
-;;      (browse-url-maybe-new-window): New arg.  Callers changed.
-;;1998-12-29  Markus Rost  <markus.rost@mathematik.uni-regensburg.de>
-;;      * browse-url.el (browse-url-mail): Give argument SEND-ACTION
-;;      to compose-mail in the correct places and as a list.
-;;1999-01-06 Markus Rost  <markus.rost@mathematik.uni-regensburg.de>
-;;      * browse-url.el (browse-url-w3): Give w3-fetch-other-window the
-;;      argument.
-;;1999-01-06  Dave Love  <fx@gnu.org>
-;;      * browse-url.el: Require (noerror) w3-auto when compiling.
-;;      (browse-url-maybe-new-window): Make it a macro and revert last
-;;      change to callers.
-;;      (browse-url-w3): Require w3 for w3-fetch-other-window.
-;;1999-01-13  Dave Love  <fx@gnu.org>
-;;      * browse-url.el (browse-url-netscape): Fix ineffective "-noraise".
-;;1999-01-19  Dave Love  <fx@gnu.org>
-;;      * browse-url.el (browse-url-maybe-new-window): Delete macro and
-;;      its uses.
-;;1999-01-25  Dave Love  <fx@gnu.org>
-;;      * browse-url.el (browse-url-netscape-program): Doc addition.
-;;1999-05-2  Andrew Innes  <andrewi@gnu.org>
-;;      * browse-url.el (browse-url-browser-function): Default to
-;;      browse-url-default-windows-browser on windows-nt.
-;;      (browse-url-default-windows-browser): New function.
-;;1999-05-02  Dave Love  <fx@gnu.org>
-;;      * browse-url.el (browse-url-browser-function): Add :version.
-;;1999-11-14  Matt Curtin <cmcurtin@interhack.net>
-;;      * Merge from GNU Emacs 20.4's browse-url.el with XEmacs.
-;;      Changes from FSF are for sensibility in XEmacs:
-;;      Replaced use of temporary-file-directory with call to
-;;      `temp-directory', change requires considering differences in
-;;      autoloading, use gnuclient instead of gnudoit.
-;;1999-12-17 Neal Becker <nbecker@fred.net>
-;;      * Add kfm support
-;;2002-11-16 Nix <nix@esperi.demon.co.uk>
-;;      * Add w3m support
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Code:
 
@@ -563,9 +244,11 @@
 
 ;;;###autoload
 (defcustom browse-url-browser-function
-  (if (memq system-type '(windows-nt ms-dos cygwin32)) ;; XEmacs change
-      'browse-url-default-windows-browser
-    'browse-url-default-browser)
+  (cond
+   ((memq system-type '(windows-nt ms-dos cygwin32)) ;; XEmacs change
+    'browse-url-default-windows-browser)
+   ((memq system-type '(darwin)) 'browse-url-default-macosx-browser)
+   (t 'browse-url-default-browser))
   "*Function to display the current buffer in a WWW browser.
 This is used by the `browse-url-at-point', `browse-url-at-mouse', and
 `browse-url-of-file' commands.
@@ -584,6 +267,7 @@ regexp should probably be \".\" to specify a default browser."
 			 :value  browse-url-w3-gnudoit)
 	  (function-item :tag "Mozilla" :value  browse-url-mozilla)
 	  (function-item :tag "Galeon" :value  browse-url-galeon)
+	  (function-item :tag "Epiphany" :value  browse-url-epiphany)
 	  (function-item :tag "Netscape" :value  browse-url-netscape)
 	  (function-item :tag "Mosaic" :value  browse-url-mosaic)
 	  (function-item :tag "Mosaic using CCI" :value  browse-url-cci)
@@ -600,6 +284,8 @@ regexp should probably be \".\" to specify a default browser."
 			 :value browse-url-generic)
 	  (function-item :tag "Default Windows browser"
 			 :value browse-url-default-windows-browser)
+	  (function-item :tag "Default Mac OS X browser"
+			 :value browse-url-default-macosx-browser)
 	  (function-item :tag "GNOME invoking Mozilla"
 			 :value browse-url-gnome-moz)
 	  (function-item :tag "Default browser"
@@ -679,6 +365,23 @@ Defaults to the value of `browse-url-galeon-arguments' at the time
   :type '(repeat (string :tag "Argument"))
   :group 'browse-url)
 
+(defcustom browse-url-epiphany-program "epiphany"
+  "*The name by which to invoke Epiphany."
+:type 'string
+:group 'browse-url)
+
+(defcustom browse-url-epiphany-arguments nil
+  "*A list of strings to pass to Epiphany as arguments."
+:type '(repeat (string :tag "Argument"))
+:group 'browse-url)
+
+(defcustom browse-url-epiphany-startup-arguments browse-url-epiphany-arguments
+  "*A list of strings to pass to Epiphany when it starts up.
+Defaults to the value of `browse-url-epiphany-arguments' at the time
+`browse-url' is loaded."
+:type '(repeat (string :tag "Argument"))
+:group 'browse-url)
+
 (defcustom browse-url-mozilla-new-window-is-tab nil
   "*Whether to open up new windows in a tab or a new window.
 If non-nil, then open the URL in a new tab rather than a new window if
@@ -693,6 +396,13 @@ If non-nil, then open the URL in a new tab rather than a new window if
   :type 'boolean
   :group 'browse-url)
 
+(defcustom browse-url-epiphany-new-window-is-tab nil
+  "*Whether to open up new windows in a tab or a new window.
+If non-nil, then open the URL in a new tab rather than a new window if
+`browse-url-epiphany' is asked to open it in a new window."
+:type 'boolean
+:group 'browse-url)
+
 ;;;###autoload
 (defcustom browse-url-new-window-flag nil
   "*If non-nil, always open a new browser window with appropriate browsers.
@@ -705,7 +415,7 @@ commands reverses the effect of this variable.  Requires Netscape version
 ;; XEmacs backwards compatibility
 ;;;###autoload
 (defvaralias 'browse-url-new-window-p 'browse-url-new-window-flag)
-
+   
 (defcustom browse-url-mosaic-program "xmosaic"
   "*The name by which to invoke Mosaic (or mMosaic)."
   :type 'string
@@ -736,8 +446,8 @@ commands reverses the effect of this variable.  Requires Netscape version
 Any substring of a filename matching one of the REGEXPs is replaced by
 the corresponding STRING using `replace-match', not treating STRING
 literally.  All pairs are applied in the order given.  The default
-value converts ange-ftp/EFS-style paths into ftp URLs and prepends
-`file:' to any path beginning with `/'.
+value converts ange-ftp/EFS-style file names into ftp URLs and prepends
+`file:' to any file name beginning with `/'.
 
 For example, adding to the default a specific translation of an ange-ftp
 address to an HTTP URL:
@@ -770,15 +480,6 @@ file rather than displaying a cached copy."
   :options '(browse-url-netscape-reload)
   :group 'browse-url)
 
-(defvar browse-url-usr1-signal
-  (if (and (boundp 'emacs-major-version)
-	   (or (> emacs-major-version 19) (>= emacs-minor-version 29)))
-      'SIGUSR1 ; Why did I think this was in lower case before?
-    30)					; Check /usr/include/signal.h.
-  "The argument to `signal-process' for sending SIGUSR1 to XMosaic.
-Emacs 19.29 accepts 'SIGUSR1, earlier versions require an integer
-which is 30 on SunOS and 16 on HP-UX and Solaris.")
-
 (defcustom browse-url-CCI-port 3003
   "*Port to access XMosaic via CCI.
 This can be any number between 1024 and 65535 but must correspond to
@@ -1093,6 +794,10 @@ to use."
       (mswindows-shell-execute "open" url)
     (w32-shell-execute "open" url)))
 
+(defun browse-url-default-macosx-browser (url &optional new-window)
+  (interactive (browse-url-interactive-arg "URL: "))
+  (start-process (concat "open " url) nil "open" url))
+
 ;; --- Netscape ---
 
 (defun browse-url-process-environment ()
@@ -1129,11 +834,11 @@ When called non-interactively, optional second argument NEW-WINDOW is
 used instead of `browse-url-new-window-flag'.
 
 The order attempted is emacs-w3m, gnome-moz-remote, Mozilla, Galeon,
-Konqueror,Netscape, Opera, Mosaic, IXI Mosaic, Lynx in an xterm, MMM,
+Konqueror, Netscape, Opera, Mosaic, IXI Mosaic, Lynx in an xterm, MMM,
 and then W3."
   (apply
     (cond
-     ((executable-find "gnome-moz-remote") 'browse-url-gnome-moz)
+     ((executable-find browse-url-gnome-moz-program) 'browse-url-gnome-moz)
      ((executable-find browse-url-mozilla-program) 'browse-url-mozilla)
      ((executable-find browse-url-galeon-program) 'browse-url-galeon)
      ((executable-find browse-url-kde-program) 'browse-url-kde)
@@ -1207,7 +912,7 @@ How depends on `browse-url-netscape-version'."
 				"reload")))
 
 (defun browse-url-netscape-send (command)
-  "Send a remote control COMMAND to Netscape."
+  "Send a remote control command to Netscape."
   (let* ((process-environment (browse-url-process-environment)))
     (apply 'start-process "netscape" nil
            browse-url-netscape-program
@@ -1260,8 +965,8 @@ requested URL in a new window."
                           (if (browse-url-maybe-new-window
                                new-window)
                               (if browse-url-mozilla-new-window-is-tab
-                                  ",new-tab")
-                            ",new-window")
+                                  ",new-tab"
+                                ",new-window"))
                           ")")))))))
     (set-process-sentinel process
 			  `(lambda (process change)
@@ -1302,7 +1007,8 @@ used instead of `browse-url-new-window-flag'."
 	       (format "%%%x" (string-to-char (match-string 0 url))) t t url)))
   (let* ((process-environment (browse-url-process-environment))
          (process (apply 'start-process
-			 (concat "galeon " url) nil
+			 (concat "galeon " url)
+			 nil
 			 browse-url-galeon-program
 			 (append
 			  browse-url-galeon-arguments
@@ -1326,8 +1032,57 @@ used instead of `browse-url-new-window-flag'."
 	       browse-url-galeon-program
 	       (append browse-url-galeon-startup-arguments (list url))))))
 
-;; GNOME means of invoking either Mozilla or Netrape.
+(defun browse-url-epiphany (url &optional new-window)
+  "Ask the Epiphany WWW browser to load URL.
+Default to the URL around or before point.  The strings in variable
+`browse-url-galeon-arguments' are also passed to Epiphany.
 
+When called interactively, if variable `browse-url-new-window-flag' is
+non-nil, load the document in a new Epiphany window, otherwise use a
+random existing one.  A non-nil interactive prefix argument reverses
+the effect of `browse-url-new-window-flag'.
+
+If `browse-url-epiphany-new-window-is-tab' is non-nil, then whenever a
+document would otherwise be loaded in a new window, it is loaded in a
+new tab in an existing window instead.
+
+When called non-interactively, optional second argument NEW-WINDOW is
+used instead of `browse-url-new-window-flag'."
+  (interactive (browse-url-interactive-arg "URL: "))
+  ;; URL encode any `confusing' characters in the URL.  This needs to
+  ;; include at least commas; presumably also close parens.
+  (while (string-match "[,)]" url)
+    (setq url (replace-match
+	       (format "%%%x" (string-to-char (match-string 0 url))) t t url)))
+  (let* ((process-environment (browse-url-process-environment))
+         (process (apply 'start-process
+			 (concat "epiphany " url)
+			 nil
+			 browse-url-epiphany-program
+			 (append
+			  browse-url-epiphany-arguments
+                          (if (browse-url-maybe-new-window new-window)
+			      (if browse-url-epiphany-new-window-is-tab
+				  '("--new-tab")
+				'("--new-window" "--noraise"))
+                            '("--existing"))
+                          (list url)))))
+    (set-process-sentinel process
+			  `(lambda (process change)
+			     (browse-url-epiphany-sentinel process ,url)))))
+
+(defun browse-url-epiphany-sentinel (process url)
+  "Handle a change to the process communicating with Epiphany."
+  (or (eq (process-exit-status process) 0)
+      (let* ((process-environment (browse-url-process-environment)))
+	;; Epiphany is not running - start it
+	(message "Starting Epiphany...")
+	(apply 'start-process (concat "epiphany " url) nil
+	       browse-url-epiphany-program
+	       (append browse-url-epiphany-startup-arguments (list url))))))
+
+;; GNOME means of invoking either Mozilla or Netscape.
+(defvar browse-url-gnome-moz-program "gnome-moz-remote")
 (defcustom browse-url-gnome-moz-arguments '()
   "*A list of strings passed to the GNOME mozilla viewer as arguments."
   :type '(repeat (string :tag "Argument"))
@@ -1349,7 +1104,7 @@ used instead of `browse-url-new-window-flag'."
   (interactive (browse-url-interactive-arg "URL: "))
   (apply 'start-process (concat "gnome-moz-remote " url)
 	 nil
-	 "gnome-moz-remote"
+	 browse-url-gnome-moz-program
 	 (append
 	  browse-url-gnome-moz-arguments
 	  (if (browse-url-maybe-new-window new-window)
@@ -1395,7 +1150,7 @@ used instead of `browse-url-new-window-flag'."
 	  (kill-buffer nil)
 	  ;; Send signal SIGUSR to Mosaic
 	  (message "Signalling Mosaic...")
-	  (signal-process pid browse-url-usr1-signal)
+	  (signal-process pid 'SIGUSR1)
 	  ;; Or you could try:
 	  ;; (call-process "kill" nil 0 nil "-USR1" (int-to-string pid))
 	  (message "Signalling Mosaic...done")
@@ -1563,7 +1318,7 @@ non-nil, load the document in a new lynx in a new term window,
 otherwise use any existing one.  A non-nil interactive prefix argument
 reverses the effect of `browse-url-new-window-flag'.
 
-When called non-interactively, optional second argument NEW-BUFFER is
+When called non-interactively, optional second argument NEW-WINDOW is
 used instead of `browse-url-new-window-flag'."
   (interactive (browse-url-interactive-arg "Lynx URL: "))
   (let* ((system-uses-terminfo t)       ; Lynx uses terminfo
