@@ -1,6 +1,6 @@
 ;;; sendmail.el --- mail sending commands for Emacs.  -*- byte-compile-dynamic: t -*-
 
-;; Copyright (C) 1985, 86, 92, 93, 94, 95, 96, 98, 2000, 2001, 2002
+;; Copyright (C) 1985, 86, 92, 93, 94, 95, 96, 98, 2000, 2001, 2002, 2003
 ;;   Free Software Foundation, Inc.
 
 ;; Maintainer: FSF
@@ -23,7 +23,7 @@
 ;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 ;; Boston, MA 02111-1307, USA.
 
-;;; Synched up with: Emacs 21.0.106
+;;; Synched up with: Emacs 21.3
 
 ;;; Commentary:
 
@@ -284,7 +284,7 @@ If a string, that string is inserted.
   which is the standard way to delimit a signature in a message.)
 Otherwise, it should be an expression; it is evaluated
 and should insert whatever you want to insert."
-  :type '(choice (const "None" nil)
+  :type '(choice (const :tag "None" nil)
 		 (const :tag "Use `.signature' file" t)
 		 (string :tag "String to insert")
 		 (sexp :tag "Expression to evaluate"))
@@ -296,6 +296,15 @@ and should insert whatever you want to insert."
   :type 'file
   :group 'sendmail)
 
+;;;###autoload
+(defcustom mail-default-directory "~/"
+  "*Directory for mail buffers.
+Value of `default-directory' for mail buffers.
+This directory is used for auto-save files of mail buffers."
+  :type '(directory :tag "Directory")
+  :group 'sendmail
+  :version "21.4")
+
 (defvar mail-reply-action nil)
 (defvar mail-send-actions nil
   "A list of actions to be performed upon successful sending of a message.")
@@ -328,6 +337,16 @@ for the recipient, who may not know how to decode them properly."
   :type '(choice (const t) (const nil) (const query) (const mime))
   :group 'sendmail)
 
+(defcustom mail-use-dsn nil
+  "*Ask MTA for notification of failed, delayed or successful delivery.
+Note that only some MTAs (currently only recent versions of Sendmail)
+support Delivery Status Notification."
+  :group 'sendmail
+  :type '(repeat (radio (const :tag "Failure" failure)
+			(const :tag "Delay" delay)
+			(const :tag "Success" success)))
+  :version "21.4")
+
 ;; Note: could use /usr/ucb/mail instead of sendmail;
 ;; options -t, and -v if not interactive.
 (defvar mail-mailer-swallows-blank-line
@@ -359,13 +378,12 @@ actually occur.")
   :type 'boolean
   :group 'sendmail)
 
-(defvar mail-mode-syntax-table nil
-  "Syntax table used while in mail mode.")
-
-(if (not mail-mode-syntax-table)
-    (progn
-     (setq mail-mode-syntax-table (copy-syntax-table text-mode-syntax-table))
-     (modify-syntax-entry ?% ". " mail-mode-syntax-table)))
+(defvar mail-mode-syntax-table
+  (let ((st (make-syntax-table)))
+    ;; define-derived-mode will make it inherit from text-mode-syntax-table.
+    (modify-syntax-entry ?% ". " st)
+    st)
+  "Syntax table used while in `mail-mode'.")
 
 (defvar mail-font-lock-keywords
   (eval-when-compile
@@ -391,7 +409,7 @@ actually occur.")
 	       (beginning-of-line) (end-of-line)
 	       (2 font-lock-constant-face nil t)
 	       (4 font-lock-comment-face nil t)))
-	    '("^\\(X-[A-Za-z0-9-]+\\|In-reply-to\\):.*"
+	    '("^\\(X-[A-Za-z0-9-]+\\|In-reply-to\\):.*\\(\n[ \t]+.*\\)*$"
 	      . font-lock-string-face))))
   "Additional expressions to highlight in Mail mode.")
 (put 'mail-mode 'font-lock-defaults '(mail-font-lock-keywords t))
@@ -529,8 +547,9 @@ actually occur.")
   :type 'hook
   :options '(footnote-mode))
 
+(defvar mail-mode-abbrev-table text-mode-abbrev-table)
 ;;;###autoload
-(defun mail-mode ()
+(define-derived-mode mail-mode text-mode "Mail"
   "Major mode for editing mail to be sent.
 Like Text Mode but with these additional commands:
 \\[mail-send]  mail-send (send the message)    \\[mail-send-and-exit]  mail-send-and-exit
@@ -545,15 +564,8 @@ Here are commands that move to a header field (and create it if there isn't):
 \\[mail-sent-via]  mail-sent-via (add a Sent-via field for each To or CC).
 Turning on Mail mode runs the normal hooks `text-mode-hook' and
 `mail-mode-hook' (in that order)."
-  (interactive)
-  (kill-all-local-variables)
   (make-local-variable 'mail-reply-action)
   (make-local-variable 'mail-send-actions)
-  (set-syntax-table mail-mode-syntax-table)
-  (use-local-map mail-mode-map)
-  (setq local-abbrev-table text-mode-abbrev-table)
-  (setq major-mode 'mail-mode)
-  (setq mode-name "Mail")
   (setq buffer-offer-save t)
   (setq mail-reply-action nil)
   (make-local-variable 'mail-send-actions)
@@ -568,12 +580,12 @@ Turning on Mail mode runs the normal hooks `text-mode-hook' and
   (set (make-local-variable 'comment-start) mail-yank-prefix)
   (make-local-variable 'adaptive-fill-regexp)
   (setq adaptive-fill-regexp
-	(concat adaptive-fill-regexp
-		"\\|[ \t]*[-[A-Za-z0-9]]*>+[ \t]*"))
+	(concat "[ \t]*[-[A-Za-z0-9]]+>+[ \t]*\\|"
+		adaptive-fill-regexp))
   (make-local-variable 'adaptive-fill-first-line-regexp)
   (setq adaptive-fill-first-line-regexp
-	(concat adaptive-fill-first-line-regexp
-		"\\|[ \t]*[-[A-Za-z0-9]]*>+[ \t]*"))
+	(concat "[ \t]*[-[A-Za-z0-9]]*>+[ \t]*\\|"
+		adaptive-fill-first-line-regexp))
   (turn-on-auto-fill)	; XEmacs - maybe filladapt should be default, too.
   ;; `-- ' precedes the signature.  `-----' appears at the start of the
   ;; lines that delimit forwarded messages.
@@ -586,9 +598,7 @@ Turning on Mail mode runs the normal hooks `text-mode-hook' and
 				page-delimiter))
   (setq paragraph-separate paragraph-start)
   (if (featurep 'menubar)
-      (easy-menu-add mail-menubar-menu))
-  (run-hooks 'text-mode-hook 'mail-mode-hook))
-
+      (easy-menu-add mail-menubar-menu)))
 
 (defun mail-header-end ()
   "Return the buffer location of the end of headers, as a number."
@@ -639,7 +649,7 @@ If within the headers, this makes the new lines into continuation lines."
 (defun mail-mode-fill-paragraph (arg)
   ;; Do something special only if within the headers.
   (if (< (point) (mail-header-end))
-      (let (beg end fieldname) 
+      (let (beg end fieldname)
 	(when (prog1 (re-search-backward "^[-a-zA-Z]+:" nil 'yes)
 		(setq beg (point)))
 	(setq fieldname
@@ -676,8 +686,6 @@ If within the headers, this makes the new lines into continuation lines."
     nil
   (setq mail-mode-map (make-sparse-keymap))
   ;; XEmacs: (define-key mail-mode-map "\M-\t" 'mail-complete)
-  (set-keymap-parents mail-mode-map (list text-mode-map)) ;; XEmacs
-  (set-keymap-name mail-mode-map 'mail-mode-map) ;; XEmacs
   (define-key mail-mode-map "\C-c?" 'describe-mode)
   (define-key mail-mode-map "\C-c\C-f\C-t" 'mail-to)
   (define-key mail-mode-map "\C-c\C-f\C-b" 'mail-bcc)
@@ -688,6 +696,7 @@ If within the headers, this makes the new lines into continuation lines."
   (define-key mail-mode-map "\C-c\C-t" 'mail-text)
   (define-key mail-mode-map "\C-c\C-y" 'mail-yank-original)
   (define-key mail-mode-map "\C-c\C-r" 'mail-yank-region)
+  ;; XEmacs: (define-key mail-mode-map [remap split-line] 'mail-split-line)
   (define-key mail-mode-map "\C-c\C-q" 'mail-fill-yanked-message)
   (define-key mail-mode-map "\C-c\C-w" 'mail-signature)
   ;; XEmacs: (define-key mail-mode-map "\C-c\C-v" 'mail-sent-via)
@@ -855,10 +864,11 @@ external program defined by `sendmail-program'."
 		    (generate-new-buffer " sendmail errors")
 		  0))
 	(tembuf (generate-new-buffer " sendmail temp"))
+	;; XEmacs: comment out
+;	(multibyte enable-multibyte-characters)
 	(case-fold-search nil)
-	(coding (and (local-variable-p 'buffer-file-coding-system
-				       (current-buffer))
-		     buffer-file-coding-system))
+	;; XEmacs: comment out
+;	(selected-coding (select-message-coding-system))
 ;;;	resend-to-addresses
 	delimline
 	fcc-was-found
@@ -868,14 +878,18 @@ external program defined by `sendmail-program'."
 		   "/usr/lib/sendmail"))
 	;; Examine these variables now, so that
 	;; local binding in the mail buffer will take effect.
-	(envelope-from (or (mail-envelope-from) (user-mail-address)))) ; XEmacs
+	(envelope-from
+	 (and mail-specify-envelope-from
+	      ;; XEmacs: use (user-mail-address)
+	      (or (mail-envelope-from) (user-mail-address)))))
     (unwind-protect
 	(save-excursion
 	  (set-buffer tembuf)
 	  (erase-buffer)
+	  ;; XEmacs: commented out
+; 	  (unless multibyte
+; 	    (set-buffer-multibyte nil))
 	  (insert-buffer-substring mailbuf)
-	  (if (or (featurep 'mule) (featurep 'file-coding)) ;; XEmacs
-	      (set-buffer-file-coding-system coding))
 	  (goto-char (point-max))
 	  ;; require one newline at the end.
 	  (or (= (preceding-char) ?\n)
@@ -973,7 +987,7 @@ external program defined by `sendmail-program'."
 			     ;; ... then undo escaping of matching parentheses,
 			     ;; including matching nested parentheses.
 			     (goto-char fullname-start)
-			     (while (re-search-forward 
+			     (while (re-search-forward
 				     "\\(\\=\\|[^\\]\\(\\\\\\\\\\)*\\)\\\\(\\(\\([^\\]\\|\\\\\\\\\\)*\\)\\\\)"
 				     fullname-end 1)
 			       (replace-match "\\1(\\3)" t)
@@ -984,6 +998,22 @@ external program defined by `sendmail-program'."
 			((eq mail-from-style 'system-default)
 			 nil)
 			(t (error "Invalid value for `mail-from-style'")))))
+	    ;; XEmacs: commented out #### why?
+; 	    ;; Possibly add a MIME header for the current coding system
+; 	    (let (charset)
+; 	      (goto-char (point-min))
+; 	      (and (eq mail-send-nonascii 'mime)
+; 		   (not (re-search-forward "^MIME-version:" delimline t))
+; 		   (progn (skip-chars-forward "\0-\177")
+; 			  (/= (point) (point-max)))
+; 		   selected-coding
+; 		   (setq charset
+; 			 (coding-system-get selected-coding 'mime-charset))
+; 		   (goto-char delimline)
+; 		   (insert "MIME-version: 1.0\n"
+; 			   "Content-type: text/plain; charset="
+; 			   (symbol-name charset) "\n"
+; 			   "Content-Transfer-Encoding: 8bit\n")))
 	    ;; Insert an extra newline if we need it to work around
 	    ;; Sun's bug that swallows newlines.
 	    (goto-char (1+ delimline))
@@ -1009,14 +1039,16 @@ external program defined by `sendmail-program'."
 \\|^resent-cc:\\|^resent-bcc:"
 				   delimline t))
 	      (let* ((default-directory "/")
-		     (args 
+		     ;; XEmacs: commented out #### why?
+;		     (coding-system-for-write selected-coding)
+		     (args
 		      (append (list (point-min) (point-max)
 				    program
 				    nil errbuf nil "-oi")
 			      ;; Always specify who from,
 			      ;; since some systems have
 			      ;; broken sendmails.
-			      (and mail-specify-envelope-from
+			      (and envelope-from
 				   (list "-f" envelope-from))
 ;;; 			      ;; Don't say "from root" if running under su.
 ;;; 			      (and (equal (user-real-login-name) "root")
@@ -1038,6 +1070,9 @@ external program defined by `sendmail-program'."
 ;;;			      (or resend-to-addresses
 				  '("-t")
 ;;;				  )
+			      (if mail-use-dsn
+				  (list "-N" (mapconcat 'symbol-name
+							mail-use-dsn ",")))
 			      )
 		      )
 		     (exit-value (apply 'call-process-region args)))
@@ -1250,7 +1285,8 @@ external program defined by `sendmail-program'."
     ;; put a marker at the end of the header
     (let ((end (copy-marker (mail-header-end)))
 	  (case-fold-search t)
-	  to-line)
+	  ;to-line
+	  )
       (goto-char (point-min))
       ;; search for the To: lines and make Sent-via: lines from them
       ;; search for the next To: line
@@ -1302,7 +1338,7 @@ external program defined by `sendmail-program'."
       (mail-position-on-field "to"))
   (insert "\nFCC: " folder))
 
-(defun mail-reply-to ()      
+(defun mail-reply-to ()
   "Move point to end of Reply-To-field.  Create a Reply-To field if none."
   (interactive)
   (if (interactive-p) (expand-abbrev))  ; XEmacs: for mail-abbrevs
@@ -1332,7 +1368,7 @@ external program defined by `sendmail-program'."
   (goto-char (mail-text-start)))
 
 (defun mail-signature (&optional atpoint)
-  "Sign letter with contents of the file `mail-signature-file'.
+  "Sign letter with signature based on `mail-signature-file'.
 Prefix arg means put contents at point."
   (interactive "P")
   (save-excursion
@@ -1341,9 +1377,11 @@ Prefix arg means put contents at point."
     (skip-chars-backward " \t\n")
     (end-of-line)
     (or atpoint
-        (delete-region (point) (point-max)))
-    (insert "\n\n-- \n")
-    (insert-file-contents (expand-file-name mail-signature-file))))
+	(delete-region (point) (point-max)))
+    (if (stringp mail-signature)
+	(insert mail-signature)
+      (insert "\n\n-- \n")
+      (insert-file-contents (expand-file-name mail-signature-file)))))
 
 (defun mail-fill-yanked-message (&optional justifyp)
   "Fill the paragraphs of a message yanked into this one.
@@ -1489,6 +1527,16 @@ and don't delete any header fields."
 	       (if mail-yank-hooks
 		   (run-hooks 'mail-yank-hooks)
 		 (mail-indent-citation))))))))
+
+(defun mail-split-line ()
+  "Split current line, moving portion beyond point vertically down.
+If the current line has `mail-yank-prefix', insert it on the new line."
+  (interactive "*")
+  (split-line
+   ;; XEmacs: #### fix this
+   ;mail-yank-prefix
+   ))
+
 
 (defun mail-attach-file (&optional file)
   "Insert a file at the end of the buffer, with separator lines around it."
@@ -1559,6 +1607,7 @@ The seventh argument ACTIONS is a list of actions to take
  when the message is sent, we apply FUNCTION to ARGS.
  This is how Rmail arranges to mark messages `answered'."
   (interactive "P")
+  ;; XEmacs: major changes here
   (if mail-use-multiple-buffers-p
 ;;; This is commented out because I found it was confusing in practice.
 ;;; It is easy enough to rename *mail* by hand with rename-buffer
@@ -1606,10 +1655,11 @@ The seventh argument ACTIONS is a list of actions to take
 	      t)))
 
     (pop-to-buffer "*mail*")
-    ;; Put the auto-save file in the home dir
-    ;; to avoid any danger that it can't be written.
-    (if (file-exists-p (expand-file-name "~/"))
-	(setq default-directory (expand-file-name "~/")))
+    ;; Avoid danger that the auto-save file can't be written.
+    (let ((dir (expand-file-name
+		(file-name-as-directory mail-default-directory))))
+      (if (file-exists-p dir)
+	  (setq default-directory dir)))
     ;; Only call auto-save-mode if necessary, to avoid changing auto-save file.
     (if (or (and auto-save-default (not buffer-auto-save-file-name))
 	    (and (not auto-save-default) buffer-auto-save-file-name))
@@ -1642,22 +1692,123 @@ The seventh argument ACTIONS is a list of actions to take
 	  (message "Auto save file for draft message exists; consider M-x mail-recover"))
       initialized)))
 
-(defun mail-recover ()
-  "Reread contents of current buffer from its last auto-save file."
+(defun mail-recover-1 ()
+  "Pop up a list of auto-saved draft messages so you can recover one of them."
   (interactive)
-  (let ((file-name (make-auto-save-file-name)))
-    (cond ((save-window-excursion
-	     (if (not (eq system-type 'vax-vms))
-		 (with-output-to-temp-buffer "*Directory*"
-		   (buffer-disable-undo standard-output)
-		   (let ((default-directory "/"))
-		     (call-process
-		      "ls" nil standard-output nil "-l" file-name))))
-	     (yes-or-no-p (format "Recover auto save file %s? " file-name)))
-	   (let ((buffer-read-only nil))
-	     (erase-buffer)
-	     (insert-file-contents file-name nil)))
-	  (t (error "mail-recover cancelled")))))
+  (let ((file-name (make-auto-save-file-name))
+	(ls-lisp-support-shell-wildcards t)
+	non-random-len wildcard)
+    ;; Remove the random part from the auto-save-file-name, and
+    ;; create a wildcard which matches possible candidates.
+    ;; Note: this knows that make-auto-save-file-name appends
+    ;; "#<RANDOM-STUFF>#" to the buffer name, where RANDOM-STUFF
+    ;; is the result of (make-temp-name "").
+    (setq non-random-len
+	  (- (length file-name) (length (make-temp-name "")) 1))
+    (setq wildcard (concat (substring file-name 0 non-random-len) "*"))
+    (if (null (file-expand-wildcards wildcard))
+	(message "There are no auto-saved drafts to recover")
+      ;; Bind dired-trivial-filenames to t because all auto-save file
+      ;; names are normally ``trivial'', so Dired will set point after
+      ;; all the files, at buffer bottom.  We want it on the first
+      ;; file instead.
+      (let ((dired-trivial-filenames t))
+	(dired-other-window wildcard (concat dired-listing-switches "t")))
+      (rename-buffer "*Auto-saved Drafts*" t)
+      (save-excursion
+	(goto-char (point-min))
+	(or (looking-at " Move to the draft file you want to recover,")
+	    (let ((inhibit-read-only t))
+	      ;; Each line starts with a space so that Font Lock mode
+	      ;; won't highlight the first character.
+	      (insert "\
+ Move to the draft file you want to recover, then type C-c C-c
+ to recover text of message whose composition was interrupted.
+ To browse text of a draft, type v on the draft file's line.
+
+ You can also delete some of these files;
+ type d on a line to mark that file for deletion.
+
+ List of possible auto-save files for recovery:
+
+"))))
+      (use-local-map
+       (let ((map (make-sparse-keymap)))
+	 (set-keymap-parent map (current-local-map))
+	 map))
+      (define-key (current-local-map) "v"
+	(lambda ()
+	  (interactive)
+	  (let ((coding-system-for-read 'emacs-mule-unix))
+	    (dired-view-file))))
+      (define-key (current-local-map) "\C-c\C-c"
+	(lambda ()
+	  (interactive)
+	  (let ((fname (dired-get-filename))
+		;; Auto-saved files are written in the internal
+		;; representation, so they should be read accordingly.
+		(coding-system-for-read 'emacs-mule-unix))
+	    (switch-to-buffer-other-window "*mail*")
+	    (let ((buffer-read-only nil))
+	      (erase-buffer)
+	      (insert-file-contents fname nil)
+	      ;; insert-file-contents will set buffer-file-coding-system
+	      ;; to emacs-mule, which is probably not what they want to
+	      ;; use for sending the message.  But we don't know what
+	      ;; was its value before the buffer was killed or Emacs
+	      ;; crashed.  We therefore reset buffer-file-coding-system
+	      ;; to the default value, so that either the default does
+	      ;; TRT, or the user will get prompted for the right
+	      ;; encoding when they send the message.
+	      (setq buffer-file-coding-system
+		    default-buffer-file-coding-system))))))))
+
+(defun mail-recover ()
+  "Recover interrupted mail composition from auto-save files.
+
+If the mail buffer has a current valid auto-save file,
+the command recovers that file.  Otherwise, it displays a
+buffer showing the existing auto-saved draft messages;
+you can move to one of them and type C-c C-c to recover that one."
+  (interactive)
+  ;; In case they invoke us from some random buffer...
+  (switch-to-buffer "*mail*")
+  ;; If *mail* didn't exist, set its directory, so that auto-saved
+  ;; drafts will be found.
+  (let ((dir (expand-file-name
+	      (file-name-as-directory mail-default-directory))))
+    (if (file-exists-p dir)
+	(setq default-directory dir)))
+  (or (eq major-mode 'mail-mode)
+      (mail-mode))
+  (let ((file-name buffer-auto-save-file-name))
+    (cond ((and file-name (file-exists-p file-name))
+	   (let ((dispbuf
+		  ;; This used to invoke `ls' via call-process, but
+		  ;; dired-noselect is more portable to systems where
+		  ;; `ls' is not a standard program (it will use
+		  ;; ls-lisp instead).
+		  (dired-noselect file-name
+				  (concat dired-listing-switches "t"))))
+	     (save-selected-window
+	       (select-window (display-buffer dispbuf t))
+	       (goto-char (point-min))
+	       (forward-line 2)
+	       (dired-move-to-filename)
+	       (setq dispbuf (rename-buffer "*Directory*" t)))
+	     (if (not (yes-or-no-p
+		       (format "Recover mail draft from auto save file %s? "
+			       file-name)))
+		 (error "mail-recover cancelled")
+	       (let ((buffer-read-only nil)
+		     (buffer-coding buffer-file-coding-system)
+		     ;; Auto-save files are written in internal
+		     ;; representation of non-ASCII characters.
+		     (coding-system-for-read 'emacs-mule-unix))
+		 (erase-buffer)
+		 (insert-file-contents file-name nil)
+		 (setq buffer-file-coding-system buffer-coding)))))
+	  (t (mail-recover-1)))))
 
 ;;;###autoload
 (defun mail-other-window (&optional noerase to subject in-reply-to cc replybuffer sendactions)
